if not PriorityQueue then return end

local L = LibStub("AceLocale-3.0"):GetLocale("PriorityQueue")
local media = LibStub:GetLibrary("LibSharedMedia-3.0");
local C = PriorityQueue.constants -- Defined in PriorityQueue LUA no locale needed.

PriorityQueue.fonteffects = {
	["none"] = L["None"],
	["OUTLINE"] = L["OUTLINE"],
	["THICKOUTLINE"] = L["THICKOUTLINE"],
	["MONOCHROME"] = L["MONOCHROME"],
}

function PriorityQueue:GetOptions()
	self:GetDefaultPriorities()
	local options = { 
		name = "PriorityQueue",
		handler = PriorityQueue,
		type='group',
		childGroups ='tree',
		args = {
			frame = {
				name = L["Priority Frame"],
				type = 'group',
				order = 1, 
				args = {
					title = {
						type = 'toggle',
						name = L["Display frame title"],
						get = "priorityTitleQuery",
						set = "ActivatePriorityTitle",
						order = 22,
					},
					combopoints = {
						type = 'toggle',
						name = L["Combo Points"],
						desc = L["help_combopoints"],
						get = "combopointsQuery",
						set = "ActivateComboPoints",
						order = 23,
					},
					worldbossonly = {
						type = 'toggle',
						name = L["World Boss Only"],
						desc = L["help_worldbossonly"],
						get = "worldbossQuery",
						set = "ActivateWorldBoss",
						order = 25,
					},
					presets = {
						name = L["Preset Priorities"],
						type = 'select',
						order = 26,
						get = "GetPresetPriority",
						set = "SetPresetPriority",
						values = self.presets.priorities.name,
					},
				},
			},
			priorities = {
				name = L["Custom Priorities"],
				type = 'group',
				order = 2,
				args = {
					priority1 = {
						type = 'select',
						name = L["First Priority"],
						get = "GetFirstPriority",
						set = "SetFirstPriority",
						values = self.priorityTable.name,
						order = 26,
					},
					priority2 = {
						type = 'select',
						name = L["Second Priority"],
						get = "GetSecondPriority",
						set = "SetSecondPriority",
						values = self.priorityTable.name,
						order = 27,
					},
					priority3 = {
						type = 'select',
						name = L["Third Priority"],
						get = "GetThirdPriority",
						set = "SetThirdPriority",
						values = self.priorityTable.name,
						order = 28,
					},
					priority4 = {
						type = 'select',
						name = L["Fourth Priority"],
						get = "GetFourthPriority",
						set = "SetFourthPriority",
						values = self.priorityTable.name,
						order = 29,
					},
					priority5 = {
						type = 'select',
						name = L["Fifth Priority"],
						get = "GetFifthPriority",
						set = "SetFifthPriority",
						values = self.priorityTable.name,
						order = 30,
					},
					priority6 = {
						type = 'select',
						name = L["Sixth Priority"],
						get = "GetSixthPriority",
						set = "SetSixthPriority",
						values = self.priorityTable.name,
						order = 31,
					},
					priority7 = {
						type = 'select',
						name = L["Seventh Priority"],
						get = "GetSeventhPriority",
						set = "SetSeventhPriority",
						values = self.priorityTable.name,
						order = 32,
					},
					priority8 = {
						type = 'select',
						name = L["Eighth Priority"],
						get = "GetEighthPriority",
						set = "SetEighthPriority",
						values = self.priorityTable.name,
						order = 33,
					},
					priority9 = {
						type = 'select',
						name = L["Ninth Priority"],
						get = "GetNinthPriority",
						set = "SetNinthPriority",
						values = self.priorityTable.name,
						order = 34,
					},
					priority10 = {
						type = 'select',
						name = L["Tenth Priority"],
						get = "GetTenthPriority",
						set = "SetTenthPriority",
						values = self.priorityTable.name,
						order = 35,
					},
					threat = {
						type = 'range',
						name = L["Threat Threshold"],
						desc = L["help_threat"],
						min = 75,
						max = 110,
						step = 1,
						get = "GetThreatThreshold",
						set = "SetThreatThreshold",
						order = 40,
					},
					cooldown = {
						type = 'range',
						name = L["Cooldown Threshold"],
						desc = L["help_cooldown"],
						min = 0,
						max = 1.5,
						step = 0.05,
						get = "GetCooldownThreshold",
						set = "SetCooldownThreshold",
						order = 41,
					},
				},
			},
			warning = {
				name = L["Warning Options"],
				type = 'group',
				order = 5, 
				args = {
					show = {
						type = 'toggle',
						name = L["Use Warning Frame"],
						desc = L["help_warningframe"],
						get = "WarningFrameQuery",
						set = "ActivateWarningFrame",
						order = 1,
					},
					range = {
						type = 'toggle',
						name = L["Out of range"],
						desc = L["help_outofrange"],
						get = "rangeWarnQuery",
						set = "ActivateRangeWarn",
						order = 4,
					},
					colour = {
						type = 'color',
						name = L["Warning Msg Colour"],
						desc = L["colWarningMessage"],
						get = "getWarningColour",
						set = "setWarningColour",
						hasAlpha = true,
						order = 7,
					},
					duration = {
						type = 'range',
						name = L["Warning Message Duration"],
						min = 1,
						max = 10,
						step = .2,
						get = "GetWarningDuration",
						set = "SetWarningDuration",
						order = 8,
					},
				},
			},
			media = {
				name = L["Media Options"],
				type = 'group',
				order = 6, 
				args = {
					border = {
						type = 'select',
						name = L["Border Texture"],
						get = "GetBorderTexture",
						set = "SetBorderTexture",
						values = self.borders,
						order = 2,
					},
					titlefont = {
						type = 'select',
						name = L["Bar Font"],
						get = "GetTitleFont",
						set = "SetTitleFont",
						values = self.fonts,
						order = 4,
					},
					titlefonteffect = {
						name = L["Bar Font Effect"],
						type = "select",
						get = "GetTitleFontEffect",
						set = "SetTitleFontEffect",
						values = self.fonteffects,
						order = 5,
					},
					titlefontsize = {
						type = 'range',
						name = L["Bar Font Size"],
						min = 6,
						max = 32,
						step = 1,
						get = "GetTitleFontSize",
						set = "SetTitleFontSize",
						order = 6,
					},
					msgfont = {
						type = 'select',
						name = L["Message Font"],
						get = "GetMsgFont",
						set = "SetMsgFont",
						values = self.fonts,
						order = 7,
					},
					msgfonteffect = {
						name = L["Message Font Effect"],
						type = "select",
						get = "GetMsgFontEffect",
						set = "SetMsgFontEffect",
						values = self.fonteffects,
						order = 8,
					},
					msgfontsize = {
						type = 'range',
						name = L["Message Font Size"],
						min = 6,
						max = 32,
						step = 1,
						get = "GetMsgFontSize",
						set = "SetMsgFontSize",
						order = 9,
					},
				},
			},
			debug = {
				type = 'toggle',
				name = L["Debug mode"],
				desc = L["help_debug"],
				get = "debugQuery",
				set = "ActivateDebug",
				order = 9,
			},
			moveframes = {
				type = 'toggle',
				name = L["Move Frames"],
				desc = L["help_display"],
				get = "FrameShowQuery",
				set = "ShowHideFrame",
				order = 10,
			},
			reset = {
				type = 'execute',
				name = L["Reset"],
				desc = L["help_reset"],
				func = "ResetPriority",
				order = 11,
			},
			config = {
				type = 'execute',
				name = L["Configure Options"],
				desc = L["help_config"],
				guiHidden = true,
				func = "OpenConfig",
				order = 13,
			},
			news = {
				type = 'execute',
				name = L["Display News"],
				desc = L["help_news"],
				func = function() self:News(true); end,
				order = 14,
			},
			version = {
				type = 'execute',
				name = L["Version"],
				desc = L["help_version"],
				func = "DisplayVersion",
				order = 15,
			},
			help = {
				type = 'description',
				name = L["help"],
				guiHidden = true,
				order = 16,
			},
		},
	}
	return options
end

function PriorityQueue:ActivateDebug()
	self.db.char.debug = not self.db.char.debug
	if (self.db.char.debug) then
		self:Print(L["config_debug_on"])
	else
		self:Print(L["config_debug_off"])
	end
end

function PriorityQueue:ActivateComboPoints()
	self.db.char.priority.combopoints = not self.db.char.priority.combopoints
	if (self.db.char.priority.combopoints) then
		for index = 1, 5 do
			self.PriorityFrame.combo[index].frame:Show()
		end
		self:Print(L["config_combopoints_on"])
	else
		for index = 1, 5 do
			self.PriorityFrame.combo[index].frame:Hide()
		end
		self:Print(L["config_combopoints_off"])
	end
end

function PriorityQueue:ActivateWorldBoss()
	self.db.char.priority.worldbossonly = not self.db.char.priority.worldbossonly
	if (self.db.char.priority.worldbossonly) then
		self:Print(L["config_worldboss_on"])
	else
		self:Print(L["config_worldboss_off"])
	end
end

function PriorityQueue:ShowHideFrame()
	self.db.char.priority.moving = not self.db.char.priority.moving
	if self.db.char.priority.moving then
		self.PriorityFrame:EnableMouse(1)
		self.PriorityFrame:SetBackdropColor(0, 0, 0, 1)
		self.PriorityFrame:Show()
		self.msgFrame:EnableMouse(1)
		self.msgFrame:SetBackdropColor(0, 0, 0, 1)
		self.msgFrame:Show()
	else
		self.PriorityFrame:EnableMouse(0);
		self.PriorityFrame:SetBackdropColor(1, 1, 1, 0);
		self.PriorityFrame:Hide()
		self.msgFrame:EnableMouse(0)
		self.msgFrame:SetBackdropColor(1, 1, 1, 0);
	end
end

function PriorityQueue:FinishedMoving(var, frame)
	local point, relativeTo, relativePoint, xOffset, yOffset = frame:GetPoint();
	var.point = point
	var.relativeTo = relativeTo
	var.relativePoint = relativePoint
	var.xOffset = xOffset
	var.yOffset = yOffset
end

function PriorityQueue:FrameShowQuery()
	return self.db.char.priority.show
end

function PriorityQueue:combopointsQuery()
	return self.db.char.priority.combopoints
end

function PriorityQueue:worldbossQuery()
	return self.db.char.priority.worldbossonly
end

function PriorityQueue:debugQuery()
	return self.db.char.debug
end

function PriorityQueue:GetBorderTexture()
    return self.db.char.border
end

function PriorityQueue:GetTitleFont()
    return self.db.char.titlefont
end

function PriorityQueue:GetTitleFontSize()
    return self.db.char.titlefontsize
end

function PriorityQueue:GetTitleFontEffect()
    return self.db.char.titlefonteffect
end

function PriorityQueue:GetMsgFont()
    return self.db.char.msgfont
end

function PriorityQueue:GetMsgFontSize()
    return self.db.char.msgfontsize
end

function PriorityQueue:GetMsgFontEffect()
    return self.db.char.msgfonteffect
end

function PriorityQueue:GetWidth()
    return self.db.char.fWidth
end

function PriorityQueue:GetScale()
    return self.db.char.scale
end

function PriorityQueue:GetMSalpha()
    return self.db.char.colours.msalpha
end

function PriorityQueue:GetMSalphaFull()
    return self.db.char.colours.msalphaFull
end

function PriorityQueue:GetShieldSound()
    return self.db.char.shieldsoundname
end

function PriorityQueue:GetWeaponSound()
    return self.db.char.weaponsoundname
end

function PriorityQueue:GetMW5sound()
    return self.db.char.mw5soundname
end

function PriorityQueue:GetThreatThreshold()
    return self.db.char.threatThreshold
end

function PriorityQueue:GetCooldownThreshold()
    return self.db.char.priority.cooldown
end

function PriorityQueue:GetFSTimeLeft()
    return self.db.char.priority.fstimeleft
end

function PriorityQueue:GetSRMana()
    return self.db.char.priority.srmana
end

function PriorityQueue:GetWSMana()
    return self.db.char.priority.wsmana
end

function PriorityQueue:GetHWHealth()
    return self.db.char.priority.hwhealth
end

function PriorityQueue:GetCHInGroup()
	return self.db.char.priority.chingroup 
end

function PriorityQueue:GetRebuffTime()
    return self.db.char.warning.timeleft
end

function PriorityQueue:GetShowWFTotals()
	return self.db.char.stats.wfcalc
end

function PriorityQueue:SetCHInGroup()
	self.db.char.priority.chingroup = not self.db.char.priority.chingroup
	if (self.db.char.priority.chingroup) then
		self:Print(L["config_chingroup_on"])
	else
		self:Print(L["config_chingroup_off"])
	end
end

function PriorityQueue:SetTitleFont(info, newValue)
	self.db.char.titlefont = newValue
	self:RedrawFrames()
end

function PriorityQueue:SetTitleFontSize(info, newValue)
	self.db.char.titlefontsize = newValue
	self:RedrawFrames()
end

function PriorityQueue:SetTitleFontEffect(info, newValue)
	self.db.char.titlefonteffect = newValue
	self:RedrawFrames()
end

function PriorityQueue:SetMsgFont(info, newValue)
	self.db.char.msgfont = newValue
	self:CreateMsgFrame()
end

function PriorityQueue:SetMsgFontSize(info, newValue)
	self.db.char.msgfontsize = newValue
	self:CreateMsgFrame()
end

function PriorityQueue:SetMsgFontEffect(info, newValue)
	self.db.char.msgfonteffect = newValue
	self:CreateMsgFrame()
end

function PriorityQueue:SetBorderTexture(info, newValue)
	self.db.char.border = newValue
	local newTexture = media:Fetch("border", self.db.char.border)
	if newTexture then
		self.frameBackdrop.edgeFile = newTexture
		self:RedrawFrames()
	else
		if self.db.char.debug then
			self:Print("border texture not found. Trying to set "..self.db.char.border)
		end
	end
end

function PriorityQueue:SetThreatThreshold(info, newValue)
	self.db.char.threatThreshold = newValue
	self:Print(L["config_Threat"]..newValue)
end

function PriorityQueue:SetCooldownThreshold(info, newValue)
	self.db.char.priority.cooldown = newValue
	self:Print(L["config_Cooldown"]..newValue)
end

function PriorityQueue:SetScale(info,newValue)
	self.db.char.scale = newValue
	self.BaseFrame:SetScale(self.db.char.scale)
	self:Print("Scale set to : "..self.db.char.scale)
end

-----------------------------------------
-- Priority Choices
-----------------------------------------

function PriorityQueue:GetPresetPriority(info)
	return self.db.char.priority.name
end

function PriorityQueue:SetPresetPriority(info, priorityValue)
	if not priorityValue then
		priorityValue = "none"
	end
	self.db.char.priority.name = priorityValue
	self.db.char.priority.option = self.presets.priorities.list[priorityValue]
end

function PriorityQueue:GetFirstPriority(info)
	return self.db.char.priority.option[1] 
end

function PriorityQueue:SetFirstPriority(info, priorityValue)
	self.db.char.priority.option[1] = priorityValue
end

function PriorityQueue:GetSecondPriority(info)
	return self.db.char.priority.option[2] 
end

function PriorityQueue:SetSecondPriority(info, priorityValue)
	self.db.char.priority.option[2] = priorityValue
end

function PriorityQueue:GetThirdPriority(info)
	return self.db.char.priority.option[3] 
end

function PriorityQueue:SetThirdPriority(info, priorityValue)
	self.db.char.priority.option[3] = priorityValue
end

function PriorityQueue:GetFourthPriority(info)
	return self.db.char.priority.option[4] 
end

function PriorityQueue:SetFourthPriority(info, priorityValue)
	self.db.char.priority.option[4] = priorityValue
end

function PriorityQueue:GetFifthPriority(info)
	return self.db.char.priority.option[5] 
end

function PriorityQueue:SetFifthPriority(info, priorityValue)
	self.db.char.priority.option[5] = priorityValue
end

function PriorityQueue:GetSixthPriority(info)
	return self.db.char.priority.option[6] 
end

function PriorityQueue:SetSixthPriority(info, priorityValue)
	self.db.char.priority.option[6] = priorityValue
end

function PriorityQueue:GetSeventhPriority(info)
	return self.db.char.priority.option[7] 
end

function PriorityQueue:SetSeventhPriority(info, priorityValue)
	self.db.char.priority.option[7] = priorityValue
end

function PriorityQueue:GetEighthPriority(info)
	return self.db.char.priority.option[8] 
end

function PriorityQueue:SetEighthPriority(info, priorityValue)
	self.db.char.priority.option[8] = priorityValue
end

function PriorityQueue:GetNinthPriority(info)
	return self.db.char.priority.option[9] 
end

function PriorityQueue:SetNinthPriority(info, priorityValue)
	self.db.char.priority.option[9] = priorityValue
end

function PriorityQueue:GetTenthPriority(info)
	return self.db.char.priority.option[10] 
end

function PriorityQueue:SetTenthPriority(info, priorityValue)
	self.db.char.priority.option[10] = priorityValue
end

function PriorityQueue:priorityQuery()
	return self.db.char.priority.show
end

function PriorityQueue:ActivatePriority()
	self.db.char.priority.show = not self.db.char.priority.show
	if (self.db.char.priority.show) then
		self:Print(L["config_priority_on"])
	else
		self.PriorityFrame:Hide()
		self:Print(L["config_priority_off"])
	end
end

function PriorityQueue:priorityTitleQuery()
	return self.db.char.priority.titleshow
end

function PriorityQueue:ActivatePriorityTitle()
	self.db.char.priority.titleshow = not self.db.char.priority.titleshow
	if self.db.char.priority.titleshow then
		self.PriorityFrame.topText:Show()
		self:Print(L["config_prioritytitle_on"])
	else
		self.PriorityFrame.topText:Hide()
		self:Print(L["config_prioritytitle_off"])
	end
end

---------------
-- Warnings
---------------

function PriorityQueue:WarningFrameQuery()
	return self.db.char.warning.show
end

function PriorityQueue:ActivateWarningFrame()
	self.db.char.warning.show = not self.db.char.warning.show
	if (self.db.char.warning.show) then
		self:Print(L["config_warnframe_on"])
	else
		self:Print(L["config_warnframe_off"])
	end
end

function PriorityQueue:rangeWarnQuery()
	return self.db.char.warning.range
end

function PriorityQueue:ActivateRangeWarn()
	self.db.char.warning.range = not self.db.char.warning.range
	if (self.db.char.warning.range) then
		self:Print(L["config_rangewarn_on"])
	else
		self:Print(L["config_rangewarn_off"])
	end
end

function PriorityQueue:getWarningColour(info)
	local colours = self.db.char.warning.colour
	return colours.r, colours.g, colours.b, colours.a
end

function PriorityQueue:setWarningColour(info,r,g,b,a)
	self.db.char.warning.colour.r = r
	self.db.char.warning.colour.g = g
	self.db.char.warning.colour.b = b
	self.db.char.warning.colour.a = a
end

function PriorityQueue:GetWarningDuration(info)
	return self.db.char.warning.duration 
end

function PriorityQueue:SetWarningDuration(info, priorityValue)
	self.db.char.warning.duration = priorityValue
end

function PriorityQueue:OpenConfig()
	  if InterfaceOptionsFrame_OpenToCategory then
	    InterfaceOptionsFrame_OpenToCategory("PriorityQueue");
    else
        InterfaceOptionsFrame_OpenToFrame("PriorityQueue");
    end
end